#ifndef _SYS_STP_H
#define _SYS_STP_H
#ident	"@(#)stp.h	1.2"
/*
 * Copyright (c) 1993 Addison-Wesley Publishing Company.
 * All rights reserved.  This source code is provided
 * for its instructional value, and is not guaranteed
 * for any particular purpose.
 */

/*
 * STP header format.
 */
typedef struct stphdr {
	uchar_t		h_version;	/* protocol version */
	uchar_t		h_type;		/* packet type */
	uchar_t		h_flags;	/* see below */
	uchar_t		h_reason;	/* disconnect reason */
	ushort_t	h_sport;	/* source port */
	ushort_t	h_dport;	/* destination port */
	ushort_t	h_size;		/* number of bytes in packet */
	ushort_t	h_pad;		/* reserved */
	ulong_t		h_seqno;	/* sequence number */
} hdr_t;

/* packet types */
#define DATA	0
#define ACK	1
#define CONNECT	2
#define ACCEPT	3
#define DISCON	4
#define RESTART	5

/* flags */
#define STOP	0x01		/* if ACK, stop transmitting */
#define MORE	0x02		/* if DATA, consumer's T_MORE flag */
#define EOM	0x04		/* End Of Message */

/* disconnect reasons */
#define DIS_USER	1	/* user-initiated disconnect */
#define DIS_TIMEOUT	2	/* acknowledgement timed out */
#define DIS_NOLINK	3	/* mux unlinked */
#define DIS_NETERR	4	/* fatal network error */

/* address format */
#define NETADDRSZ	6		/* size of network address */
#define MAXNETSAPSZ	4		/* max net SAP size */
#define STPADDRSZ	(2+NETADDRSZ)	/* size of stp address */

struct stpaddr {
	ushort_t	a_port;			/* port number */
	uchar_t		a_net[NETADDRSZ];	/* network address */
};

#define MAXCREQ		10	/* max # of connect requests queued */
#define MAXPORT		65535	/* maximum port number */
#define STPVERS		1	/* protocol version 1 */

#ifdef _KERNEL

#define MAXTIDUSZ	8192	/* maximum TIDU size */
#define RETRTIME	300000	/* time to retransmit (in us.) */
#define MAXRETR		4	/* max number of retransmits */

/* transport endpoint definition */
typedef struct stp_endpoint {
	queue_t		*s_rdq;		/* read queue */
	ulong_t		s_tpstate;	/* TPI state */
	ulong_t		s_seqno;	/* next sequence number to use */
	ulong_t		s_ackno;	/* last seqno ack sent */
	ulong_t		s_rcvack;	/* last seqno ack received */
	ushort_t	s_lport;	/* local port */
	ushort_t	s_flag;		/* see below */
	ushort_t	s_nretr;	/* number of retransmits */
	ushort_t	s_msgxmitsz;	/* message transmission size */
	ushort_t	s_totxmitsz;	/* total transmitted so far */
	mblk_t		*s_rxmp;	/* message to retransmit */
	mblk_t		*s_rcvmp;	/* message being received */
	ulong_t		s_txtime;	/* time message transmitted */
	ulong_t		s_rrecid;	/* read-side allocb failures */
	ulong_t		s_wrecid;	/* write-side allocb failures */
	ushort_t	s_maxqlen;	/* max # of conninds for user */
	ushort_t	s_qlen;		/* # of conninds queued */
	ulong_t		s_conseq;	/* connect sequence counter */
	struct stpaddr s_faddr;	/* foreign address */
	mblk_t		*s_cmp[MAXCREQ];/* connect request array */
} stp_t;

/* flags */
#define RRECBCALL	0x01	/* s_rrecid is a bufcall ID */
#define RRECTOUT	0x02	/* s_rrecid is a timeout ID */
#define WRECBCALL	0x04	/* s_wrecid is a bufcall ID */
#define WRECTOUT	0x08	/* s_wrecid is a timeout ID */
#define NOXMIT		0x10	/* other side has flow-controlled */
#define FLOWCTL		0x20	/* we were flow-controlled */
#define DORESTART	0x40	/* restart other end's transmitter */

#endif /* _KERNEL */

#endif /* _SYS_STP_H */
